#[macro_use]
extern crate async_trait;
#[macro_use]
extern crate hex_literal;
#[macro_use]
extern crate serde;

mod install_package;
mod parse_package;
mod queue_utils;
mod semver;

use std::{path::PathBuf, sync::Arc, time::Duration};

use anyhow::Result;
use clap::{Parser, Subcommand};

use reqwest::Client;

use tokio::time::sleep;

use crate::install_package::{install_package, PackageDef};

pub type TaskQueue = deadqueue::limited::Queue<PackageDef>;

#[derive(Debug, Parser)]
struct Opt {
  #[clap(subcommand)]
  command: Command,
}

#[derive(Debug, Subcommand)]
enum Command {
  Install {
    #[clap(long = "dir")]
    dir: PathBuf,
  },
}

#[tokio::main]
async fn main() -> Result<()> {
  let opt = Opt::parse();
  println!("Hello, world! {opt:?}");

  let client = Client::builder()
    .use_rustls_tls()
    .pool_idle_timeout(Duration::from_secs(30))
    .build()?;

  match opt.command {
    Command::Install { dir } => {
      // Parse package.json
      let package_json_path = dir.join("package.json");
      let node_modules_path = dir.join("node_modules");
      let rget_path = node_modules_path.join(".rget");
      tokio::fs::create_dir_all(&rget_path).await?;
      // let parsed = parse_package_json(package_json_path)?;

      let queue = Arc::new(TaskQueue::new(1_000));

      // Handle the first one specifically
      let task = PackageDef::Local(package_json_path);
      // queue.push(task).await;
      install_package(queue.clone(), task, client.clone(), rget_path.clone())
        .await?;

      for _worker in 0..10 {
        let queue = queue.clone();
        let client = client.clone();
        let rget_path = rget_path.clone();
        tokio::spawn(async move {
          loop {
            let queue = queue.clone();
            let client = client.clone();
            let task = queue.pop().await;
            // println!("Handling task {task:?}");
            match install_package(queue, task, client, rget_path.clone()).await
            {
              Ok(_) => {}
              Err(err) => {
                eprintln!("WTF? {err} {err:?}");
              }
            }
            // println!("worker[{}] processing task[{}] ...", worker, task);
          }
        });
      }

      while queue.len() > 0 {
        // println!("Waiting for workers to finish...");
        sleep(Duration::from_millis(1_000)).await;
      }

      println!("Done!");
    }
  }

  Ok(())
}
