// https://github.com/npm/registry/blob/master/docs/REGISTRY-API.md

use std::{collections::HashMap, fs::File, path::Path};

use anyhow::Result;

// TODO: Optimize using owning ref?
#[derive(Debug, Deserialize)]
pub struct Package {
  pub name: String,
  pub version: String,

  #[serde(default)]
  pub dependencies: HashMap<String, String>,

  #[serde(rename = "devDependencies", default)]
  pub dev_dependencies: HashMap<String, String>,
}

#[derive(Debug, Deserialize)]
pub struct PackageMetadata {
  pub name: String,
  pub versions: HashMap<String, VersionObject>,
}

#[derive(Debug, Deserialize)]
pub struct VersionObject {
  pub dist: Dist,
}

#[derive(Debug, Deserialize)]
pub struct Dist {
  pub shasum: String,
  pub tarball: String,
}

pub fn parse_package_json(path: impl AsRef<Path>) -> Result<Package> {
  let parsed: Package = {
    let file = File::open(path)?;
    simd_json::from_reader(file)?
  };

  Ok(parsed)
}
