use anyhow::{Context, Result};
use semver::VersionReq;

use crate::{install_package::PackageDef, parse_package::Package, TaskQueue};

pub async fn add_reqs_to_queue(
  queue: &TaskQueue,
  package: &Package,
) -> Result<()> {
  for (package_name, version_req) in package.dependencies.iter() {
    let version_req = VersionReq::parse(version_req)
      .with_context(|| format!("could not parse version {version_req:?}"))?;
    queue
      .push(PackageDef::Name(package_name.to_owned(), version_req))
      .await;
  }
  Ok(())
}
